% This is the main run file for estimating the NP(freq),NP(B),NP(BM) and EW(B)
% Mitchell, Poon and Zhu (2023).
%
% This code is free to use for academic purposes only, provided that the 
% paper is cited as:
%
% Mitchell, J, Poon, A, and Zhu, D. (2024). Constructing Density Forecasts From Quantile Regressions: Multimodality in Macro-Financial Dynamics.
% Journal of Applied Econometrics, Forthcoming.
%
% This code comes without technical support of any kind.  It is expected to
% reproduce the results reported in the paper. Under no circumstances will
% the authors be held responsible for any use (or misuse) of this code in
% any way.


clear all; clc;

period = 1 ; % 1- Pre-Great Recession period, 2 - Great Recession period
fhor = 1 ; % 1 - one-quarter ahead, 2 - one-year ahead
model = 1 ; % 1 - NP(freq) , 2 - NP(B), 3 - NP(BM), 4-EW(B)
modelvar = {'NP(freq)','NP(B)','NP(BM)','EW(B)','ABG'};
tau=0.05:0.05:0.95; % no. of quantiles

if fhor==1
data = readmatrix("AER.xlsx",Sheet="model1");

if period == 1
ABG=readmatrix('ABGGFCparameters.xlsx',Sheet='onequarterin1');
perioddate = {'2005Q1','2005Q2','2005Q3','2005Q4'};
x = -5:.01:10;
else
ABG=readmatrix('ABGGFCparameters.xlsx',Sheet='onequarterin');
perioddate = {'2008Q1','2008Q2','2008Q3','2008Q4'};
x = -10:.01:10;
end


else
data = readmatrix("AER.xlsx",Sheet="model2");

if period == 1
ABG=readmatrix('ABGGFCparameters.xlsx',Sheet='oneyearin1');
perioddate = {'2005Q1','2005Q2','2005Q3','2005Q4'};
x = -5:.01:10;
else
ABG=readmatrix('ABGGFCparameters.xlsx',Sheet='oneyearin');
perioddate = {'2008Q1','2008Q2','2008Q3','2008Q4'};
x = -10:.01:10;
end

end
Y = data(:,2); % y variable
T = length(Y);
X = [ones(T,1) data(:,3:end)]; % regressors
p = size(X,2);

if model == 1
[Q_f Y_f]=QR(tau,X,Y);
sample = Y_f';
elseif model == 2
sample = BQR(20000,10000,Y,X,tau,0,1);    
elseif model == 3
sample = BQR(20000,10000,Y,X,tau,1,1);    
elseif model == 4
sample = BQR(20000,10000,Y,X,tau,0,0); 
end

%% Plot density plots

para = ABG(:,2:5);
actual = ABG(:,6);
time = ABG(:,1);

for i = 1:4
    n = time(i);
    ABGden(i,:) = dskt(x,para(i,1),para(i,2),para(i,3),para(i,4));
    [den(i,:),xi] = ksdensity(sample(:,n),x);
end


figure
subplot(2,2,1)
p = plot(x,ABGden(1,:),x,den(1,:));
xlim([x(1) 10])
p(1).LineWidth = 2;
p(2).LineWidth = 2;
k = xline(actual(1),'--k',{'Actual value'});
k(1).LineWidth = 2;
title(perioddate(1))
subplot(2,2,2)
p = plot(x,ABGden(2,:),x,den(2,:));
xlim([x(1) 10])
p(1).LineWidth = 2;
p(2).LineWidth = 2;
k = xline(actual(2),'--k',{'Actual value'});
k(1).LineWidth = 2;
title(perioddate(2))
subplot(2,2,3)
p = plot(x,ABGden(3,:),x,den(3,:));
xlim([x(1) 10])
p(1).LineWidth = 2;
p(2).LineWidth = 2;
k = xline(actual(3),'--k',{'Actual value'});
k(1).LineWidth = 2;
title(perioddate(3))
subplot(2,2,4)
p = plot(x,ABGden(4,:),x,den(4,:));
xlim([x(1) 10])
p(1).LineWidth = 2;
p(2).LineWidth = 2;
k = xline(actual(4),'--k',{'Actual value'});
k(1).LineWidth = 2;
title(perioddate(4))
legend([modelvar(5) modelvar(model)])
