% This is the main run file for estimating the NP(freq) model the expected
% loss and four moments over time.
% Mitchell, Poon and Zhu (2023).
%
% This code is free to use for academic purposes only, provided that the 
% paper is cited as:
%
% Mitchell, J, Poon, A, and Zhu, D. (2024). Constructing Density Forecasts From Quantile Regression: Multimodality in Macro-Financial Dynamics.
% Journal of Applied Econometrics, Forthcoming.
%
% This code comes without technical support of any kind.  It is expected to
% reproduce the results reported in the paper. Under no circumstances will
% the authors be held responsible for any use (or misuse) of this code in
% any way.

clear all; clc;

ls1=readmatrix('ABGGFCparameters.xlsx',Sheet='onequarter_loss');
ls2=readmatrix('ABGGFCparameters.xlsx',Sheet='oneyear_loss');
mom1=readmatrix('ABGGFCparameters.xlsx',Sheet='onequarter_mom');
mom2=readmatrix('ABGGFCparameters.xlsx',Sheet='oneyear_mom');

%% One-quarter ahead
data = readmatrix("AER.xlsx",Sheet="model1");
tau=0.05:0.05:0.95; % no. of quantiles
Y = data(:,2); % y variable
T = length(Y);
X = [ones(T,1) data(:,3:end)]; % regressors
p = size(X,2);
[Q_f Y_f]=QR(tau,X,Y);
NPsample1 = Y_f';

data = readmatrix("AER.xlsx",Sheet="model2");
tau=0.05:0.05:0.95; % no. of quantiles
Y = data(:,2); % y variable
T = length(Y);
X = [ones(T,1) data(:,3:end)]; % regressors
p = size(X,2);
[Q_f Y_f]=QR(tau,X,Y);
NPsample4 = Y_f';

[ES1,EL1]=Scores(NPsample1,.05); % Generate expected loss for one-quarter ahead
[ES2,EL2]=Scores(NPsample4,.05); % Generate expected loss for one-year ahead

date1 = 1973:.25:2015.5;
date2 = 1974:.25:2015.75;

ES3 = ls1(:,2);
EL3 = ls1(:,3);
ES4 = ls2(:,2);
EL4 = ls2(:,3);

figure
subplot(2,2,3)
plot(date1,ES3,date1,ES1,date1,zeros(length(date1),1),'r--','linewidth',1)
ylim([-20 20])
xlim([date1(1) date1(end)])
title('Expected Shortfall (one quarter ahead)')
subplot(2,2,1)
plot(date1,EL3,date1,EL1,date1,zeros(length(date1),1),'r--','linewidth',1)
ylim([-20 20])
xlim([date1(1) date1(end)])
title('Longrise (one quarter ahead)')
subplot(2,2,4)
plot(date2,ES4,date2,ES2,date2,zeros(length(date2),1),'r--','linewidth',1)
ylim([-20 20])
xlim([date2(1) date2(end)])
title('Expected Shortfall (one year ahead)')
subplot(2,2,2)
plot(date2,EL4,date2,EL2,date2,zeros(length(date2),1),'r--','linewidth',1)
ylim([-20 20])
xlim([date2(1) date2(end)])
title('Longrise (one year ahead)')
legend('ABG','NP(freq)')

%% Plot four moments
NPmom1 = [mean(NPsample1)' var(NPsample1)' skewness(NPsample1)' kurtosis(NPsample1)'];
NPmom4 = [mean(NPsample4)' var(NPsample4)' skewness(NPsample4)' kurtosis(NPsample4)'];

figure
subplot(2,2,1)
plot(date1,mom1(:,2),date1,NPmom1(:,1),'linewidth',1)
xlim([date1(1) date1(end)])
title('Mean')
subplot(2,2,2)
plot(date1,mom1(:,3),date1,NPmom1(:,2),'linewidth',1)
xlim([date1(1) date1(end)])
title('Variance')
subplot(2,2,3)
plot(date1,mom1(:,4),date1,NPmom1(:,3),'linewidth',1)
xlim([date1(1) date1(end)])
title('Skewness')
subplot(2,2,4)
plot(date1,mom1(:,5),date1,NPmom1(:,4),'linewidth',1)
xlim([date1(1) date1(end)])
title('Kurtosis')
legend('ABG','NP(freq)')
sgtitle('One quarter ahead')

figure
subplot(2,2,1)
plot(date2,mom2(:,2),date2,NPmom4(:,1),'linewidth',1)
xlim([date2(1) date2(end)])
title('Mean')
subplot(2,2,2)
plot(date2,mom2(:,3),date2,NPmom4(:,2),'linewidth',1)
xlim([date2(1) date2(end)])
title('Variance')
subplot(2,2,3)
plot(date2,mom2(:,4),date2,NPmom4(:,3),'linewidth',1)
xlim([date2(1) date2(end)])
title('Skewness')
subplot(2,2,4)
plot(date2,mom2(:,5),date2,NPmom4(:,4),'linewidth',1)
xlim([date2(1) date2(end)])
title('Kurtosis')
legend('ABG','NP(freq)')
sgtitle('One year ahead')
